/*
** Parse_Loader_Conf.c for Gmurf in ~/Gmurf/Charge_Sound/
** 
** Made by 
** Login   <ahes@ker-ys.epita.fr>
** 
** Started on  Fri Mar  5 14:46:48 1999 
** Last update Mon Mar 15 21:02:04 1999 Rmi Coupet
*/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "Errors.h"
#include "Messages.h"
#include "Constants.h"
#include "Parse_Loader_Conf.h"

/*
** is_same_type: return 1 if the type of 'file' is the same that 'type' 
*/
static	int	is_same_type(file, type)
char		*file;
char		*type;
{
  int	cpt;
  int	result;
  char	char_type;
  char	char_file;

  result = 1;
  for (cpt = 0; cpt < strlen(type); cpt++)
    {
      if ((type[cpt] >= 'a') && (type[cpt] <= 'z'))
	char_type = type[cpt] + 'A' - 'a';
      else
	char_type = type[cpt];
      if ((file[strlen(file) - strlen (type) + cpt] >= 'a') && (file[strlen(file) + strlen(type) + cpt] <= 'z'))
	char_file = file[strlen(file) - strlen(type) + cpt] + 'A' - 'a';
      else
	char_file = file[strlen(file) - strlen(type) + cpt];
      if (char_type != char_file)
	result = 0; 
    }
  return (result);
}

static	FILE	*create_new_sound_file_loader()
{
  FILE		*temp_file;

  printf("%s %s\n", FILE_NON_EXISTENT, SOUND_FILE_LOADER);
  printf("...%s\n", FILE_CREAT);
  if ((temp_file = fopen(SOUND_FILE_LOADER, "a+")) == NULL)
    printf("%s\n", BAD_CREAT);
  return (temp_file);
}

/*
** get_next_word: parse a string and return the first found word
*/
static	char	*get_next_word(buf, cpt, line)
char		*buf;
int		*cpt;
int		line;
{
  int	end;
  int	begin;
  char	*result;

  while ((buf[*cpt] != '"') && (*cpt < MAX_LINE_LENGTH))
    (*cpt)++;
  if ((*cpt == MAX_LINE_LENGTH) || (*cpt > strlen(buf)))
    {
      printf("%s:\n %s:\n   %s\n ligne %d :     %s\n", GMURF_ERROR, BAD_SCRIPT, SOUND_FILE_LOADER, line, buf);
      exit(-1);
    }
  begin = (*cpt)++ + 1;
  while ((buf[*cpt] != '"') && (*cpt < MAX_LINE_LENGTH))
    (*cpt)++;
  if ((*cpt == MAX_LINE_LENGTH) || (*cpt > strlen(buf)) || (buf[*cpt - 1] == '"'))
    {
      printf("%s:\n %s:\n   %s\n ligne %d :%s\n%s\ncpt%d\n", GMURF_ERROR, BAD_SCRIPT, SOUND_FILE_LOADER, line, buf, (buf[*cpt - 1] == '"'? EMPTY_WORD:UNTEMINATED_WORD), *cpt);
      exit(-1);
    }
  end = (*cpt);
  (*cpt)++;
  
  result = (char *) malloc((end - begin + 20) * sizeof(char)); 
  for (*cpt = begin; *cpt < end; ++(*cpt))
    result[*cpt - begin] = buf[*cpt];
  result[*cpt - begin] = '\0';
  (*cpt) += 2; 
  return (result);
}  

/*
** test_sound_file_loader: test if the file containing the sound files is good or not
**			   if the result is wrong, then exit
*/
void	test_sound_file_loader()
{
  int	cpt;
  int	line;
  FILE	*file2test;
  char	buf[MAX_LINE_LENGTH];
  char	*path;
  
  if (access(SOUND_FILE_LOADER, F_OK) == -1)
    file2test = create_new_sound_file_loader();
  else
    {
      file2test = fopen(SOUND_FILE_LOADER, "r");
      line = 0;
      while (!feof(file2test))
	{
	  fgets(buf, MAX_LINE_LENGTH, file2test);
	  line++;
	  if ((buf[0] != '#') && (strlen(buf) >= 2))
	    {
	      cpt = 0;
	      free(get_next_word(buf, &cpt, line));
	      path = get_next_word(buf, &cpt, line);
	      if(access(path, F_OK | R_OK | X_OK) == -1)
		{
		  printf("%s: %s(%s)\n  %s %d:%s", GMURF_ERROR, BAD_ACCESS, path, BAD_LINE, line, buf);
		  free(path);
		  exit(-1);
		}
	      free(path);
	    }
	}
    }
  if (file2test != NULL)
    fclose(file2test);
}

/*
** get_sound_loader_path: get the path of the plug in that read the sound-file
**			  if wrong, then return NULL
*/
char	*get_sound_loader_path(file)
char	*file;
{
  int	cpt;
  int	line;
  FILE	*pathfile;
  char	buf[MAX_LINE_LENGTH];
  char  *type;
  char  *path;

  pathfile = fopen(SOUND_FILE_LOADER, "r");
  line = 0;
  while (!feof(pathfile))
    {
      fgets(buf, MAX_LINE_LENGTH, pathfile);
      line++;
      if ((buf[0] != '#') && (strlen(buf) >= 2))
	{
	  cpt = 0;
	  type = get_next_word(buf, &cpt, line);
	  path = get_next_word(buf, &cpt, line);
	  if (is_same_type(file, type) == 1)
	    {
	      fclose(pathfile);
	      free(type);
	      return (path);
	    }
	  free(type); free(path);
	}
    }
  gmurf_errno = UNKNOWN_SOUND_TYPE;
  fclose(pathfile);
  return (NULL);
}






