/*
** Load_WAV.c for Load_WAV (Plug in for Gmurf) in ~/Gmurf/Load_WAV/
** 
** Made by 
** Login   <ahes@ker-ys.epita.fr>
** 
** Started on  Wed Feb 17 23:14:38 1999 
** Last update Sun Mar 14 01:48:02 1999 
*/

#include "Constants.h"
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include "Errors.h"
#include <string.h>
#include "Fct_Sys.h"
#include "Load_WAV.h"
#include "WavePlay.h"
#include <sys/wait.h>
#include "TempGmurf.h"
#include "WaveHeader.h"

/*
****************************************************************************************
**           FONCTIONS PROPRES AUX FICHIERS WAVES                                     **
****************************************************************************************
*/

/*
** copy_wave_file2temp: copie la partie 'onde sonore' du fichier WAV
**                      dans le fichier temporaire
*/
int copy_wave_file2temp(temp_wav, temp_file)
     t_wave_file	*temp_wav;
     int		temp_file;
{
  int	n;
  char	buffer[BUFFER_SIZE];

  n = read(temp_wav->file, &buffer, BUFFER_SIZE);
  temp_wav->processed_bytes += n;
  if (temp_wav->processed_bytes > temp_wav->header.data_size) 
    n -= (temp_wav->processed_bytes - temp_wav->header.data_size);
  if (write(temp_file, (void *) buffer, n) == -1)
    {
#ifdef TEXT_MSG
      printf ("erreur write!!!!  errno = %d, temp_file = %d", errno, temp_file);
#endif
    }
  return (n == BUFFER_SIZE);
}

/*
** charge_wav: ouvre un fichier WAV, copy ses proprietes dans 'temp', puis rempli
**             le fichier temporaire (qui est deja ouvert) avec son contenu sonore
*/
int charge_wav (wav_path, temp)
     t_tempgmurf	*temp;
{   
  t_wave_file	wave_file;

  wave_file = wave_open_for_use(temp->origname);
/*    printf("riff_str[4] = %s\n", wave_file.header.riff_str); */
/*    printf("size = %d\n", wave_file.header.size); */
/*    printf("wave_fmt_str[8] = %s\n", wave_file.header.wave_fmt_str); */
/*    printf("smt_size = %d\n", wave_file.header.smt_size); */
/*    printf("format_tag = %d\n", wave_file.header.format_tag); */
/*    printf("chanels = %d\n", wave_file.header.chanels); */
/*    printf("sample_rate = %d\n", wave_file.header.sample_rate);  */
/*    printf("avg_byte_per_sec = %d\n", wave_file.header.avg_byte_per_sec); */
/*    printf("block_align = %d\n", wave_file.header.block_align); */
/*    printf("bits_per_sample = %d\n", wave_file.header.bits_per_sample); */
/*    printf("data_str[4] = %s\n", wave_file.header.data_str);   */
/*    printf("data_size = %d\n\n", wave_file.header.data_size); */
  if (wave_file.file == -1)
    return (-1);
  temp->bits_per_sample = wave_file.header.bits_per_sample;
  temp->chanels	        = wave_file.header.chanels;
  temp->sample_rate	= wave_file.header.sample_rate;
  temp->position	= 0;
  while (copy_wave_file2temp(&wave_file, temp->file)) 
    {};							/* Mise a jour du buffer */
  temp->temp_size = give_file_size(temp->file);
  wave_close(wave_file);
  return (0);
}

void	print_error(msg)
char	*msg;
{
int	child_result;
PID_T   pid;

#ifdef TEXT_MSG
  printf("%s\n   errno = %d\n", msg, errno);
#endif
  pid = fork();
  if (pid == 0)
    {
      if(execlp(ERROR_PRINTER, ERROR_PRINTER, msg) == -1)
      {
#ifdef TEXT_MSG
	  printf("erreur lors de l'execution de execve, errno = %d\n", errno);
#endif
	}
    }
  else
    waitpid(pid, &child_result, WUNTRACED);
  exit(-1);
}

int main (argc, argv)
int	argc;
char	*argv[];
{
  int fd; 
  t_tempgmurf	*tempgmurf;

  tempgmurf = (t_tempgmurf *) malloc(sizeof(t_tempgmurf));
  if (argc != 2)
    print_error(ARGS_ERROR);
  if ((fd = open(argv[1], O_RDWR)) == -1)
    print_error(SWAP_OPEN_ERROR);
  if (read(fd, &(*tempgmurf), sizeof(t_tempgmurf)) == -1)
    print_error(SWAP_READ_ERROR);
  if (charge_wav(tempgmurf) != 0)
    print_error(WAV_OPEN_ERROR);
  close(fd);
  if (unlink(argv[1]) == -1)
    print_error(SWAP_DEL_ERROR);
  if ((fd = open(argv[1], O_CREAT | O_TRUNC | O_RDWR, 0666)) == -1)
    print_error(SWAP_MAKE_ERROR);
  if (write(fd, tempgmurf, sizeof(t_tempgmurf)) == -1)
    print_error(SWAP_WRITE_ERROR);
  close(tempgmurf->file);
  close(fd);
  free(tempgmurf);
  return(1);
}





